@@ -15,16 +15,40 @@
 namespace Doctrine\Bundle\DoctrineBundle\Tests\DependencyInjection;
 
 use Doctrine\Bundle\DoctrineBundle\DependencyInjection\DoctrineExtension;
+use Doctrine\Bundle\DoctrineBundle\Tests\Builder\BundleConfigurationBuilder;
 use Doctrine\Common\Proxy\AbstractProxyFactory;
+use Doctrine\DBAL\Connection;
 use Doctrine\ORM\Version;
+use PHPUnit\Framework\TestCase;
+use Symfony\Component\DependencyInjection\Compiler\ResolveDefinitionTemplatesPass;
 use Symfony\Component\DependencyInjection\ContainerBuilder;
 use Symfony\Component\DependencyInjection\Definition;
 use Symfony\Component\DependencyInjection\ParameterBag\ParameterBag;
 use Symfony\Component\DependencyInjection\Reference;
-use Symfony\Component\DependencyInjection\Compiler\ResolveDefinitionTemplatesPass;
 
-class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
+class DoctrineExtensionTest extends TestCase
 {
+
+    public function testDbalGenerateDefaultConnectionConfiguration()
+    {
+        $container = $this->getContainer();
+        $extension = new DoctrineExtension();
+
+        $container->registerExtension($extension);
+
+        $extension->load(array(array('dbal' => array())), $container);
+
+        // doctrine.dbal.default_connection
+        $this->assertEquals('%doctrine.default_connection%', $container->getDefinition('doctrine')->getArgument(3));
+        $this->assertEquals('default', $container->getParameter('doctrine.default_connection'));
+        $this->assertEquals('root', $container->getDefinition('doctrine.dbal.default_connection')->getArgument(0)['user']);
+        $this->assertNull($container->getDefinition('doctrine.dbal.default_connection')->getArgument(0)['password']);
+        $this->assertEquals('localhost', $container->getDefinition('doctrine.dbal.default_connection')->getArgument(0)['host']);
+        $this->assertNull($container->getDefinition('doctrine.dbal.default_connection')->getArgument(0)['port']);
+        $this->assertEquals('pdo_mysql', $container->getDefinition('doctrine.dbal.default_connection')->getArgument(0)['driver']);
+        $this->assertEquals(array(), $container->getDefinition('doctrine.dbal.default_connection')->getArgument(0)['driverOptions']);
+    }
+
     public function testDbalOverrideDefaultConnection()
     {
         $container = $this->getContainer();
@@ -162,7 +186,11 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
 
-        $extension->load(array(array('dbal' => array('connections' => array('default' => array('password' => 'foo')))), array(), array('dbal' => array('default_connection' => 'foo')), array()), $container);
+        $extension->load(array(
+            array('dbal' => array('connections' => array('default' => array('password' => 'foo')))),
+            array(),
+            array('dbal' => array('default_connection' => 'foo')),
+            array()), $container);
 
         $config = $container->getDefinition('doctrine.dbal.default_connection')->getArgument(0);
 
@@ -170,12 +198,35 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals('root', $config['user']);
     }
 
-    public function testDependencyInjectionConfigurationDefaults()
+    public function testDbalWrapperClass()
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
 
-        $extension->load(array(array('dbal' => array('connections' => array('default' => array('password' => 'foo'))), 'orm' => array('default_entity_manager' => 'default', 'entity_managers' => array('default' => array('mappings' => array('YamlBundle' => array())))))), $container);
+        $extension->load(
+            array(
+                array('dbal' => array('connections' => array(
+                    'default' => array('password' => 'foo', 'wrapper_class' => TestWrapperClass::class),
+                    'second' => array('password' => 'boo'),
+                ))),
+                array(),
+                array('dbal' => array('default_connection' => 'foo')),
+                array()
+            ),
+            $container
+        );
+
+        $this->assertEquals(TestWrapperClass::class, $container->getDefinition('doctrine.dbal.default_connection')->getClass());
+        $this->assertNull($container->getDefinition('doctrine.dbal.second_connection')->getClass());
+    }
+
+    public function testDependencyInjectionConfigurationDefaults()
+    {
+        $container = $this->getContainer();
+        $extension = new DoctrineExtension();
+        $config = BundleConfigurationBuilder::createBuilderWithBaseValues()->build();
+
+        $extension->load(array($config), $container);
 
         $this->assertFalse($container->getParameter('doctrine.orm.auto_generate_proxy_classes'));
         $this->assertEquals('Doctrine\ORM\Configuration', $container->getParameter('doctrine.orm.configuration.class'));
@@ -202,19 +253,23 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals('Doctrine\ORM\Cache\CacheConfiguration', $container->getParameter('doctrine.orm.second_level_cache.cache_configuration.class'));
         $this->assertEquals('Doctrine\ORM\Cache\RegionsConfiguration', $container->getParameter('doctrine.orm.second_level_cache.regions_configuration.class'));
 
-        $config = array(
-            'proxy_namespace' => 'MyProxies',
-            'auto_generate_proxy_classes' => true,
-            'default_entity_manager' => 'default',
-            'entity_managers' => array(
-                'default' => array(
-                    'mappings' => array('YamlBundle' => array()),
+
+        $config = BundleConfigurationBuilder::createBuilder()
+            ->addBaseConnection()
+            ->addEntityManager(array(
+                'proxy_namespace' => 'MyProxies',
+                'auto_generate_proxy_classes' => true,
+                'default_entity_manager' => 'default',
+                'entity_managers' => array(
+                    'default' => array(
+                        'mappings' => array('YamlBundle' => array()),
                     ),
                 ),
-        );
+            ))
+            ->build();
 
         $container = $this->getContainer();
-        $extension->load(array(array('dbal' => array('connections' => array('default' => array('password' => 'foo'))), 'orm' => $config)), $container);
+        $extension->load(array($config), $container);
         $this->compileContainer($container);
 
         $definition = $container->getDefinition('doctrine.dbal.default_connection');
@@ -290,28 +345,33 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
 
-        $config = array(
-            'proxy_namespace' => 'MyProxies',
-            'auto_generate_proxy_classes' => 'eval',
-            'default_entity_manager' => 'default',
-            'entity_managers' => array(
-                'default' => array(
-                    'mappings' => array('YamlBundle' => array()),
+        $config = BundleConfigurationBuilder::createBuilder()
+            ->addBaseConnection()
+            ->addEntityManager(array(
+                'proxy_namespace' => 'MyProxies',
+                'auto_generate_proxy_classes' => 'eval',
+                'default_entity_manager' => 'default',
+                'entity_managers' => array(
+                    'default' => array(
+                        'mappings' => array('YamlBundle' => array()),
+                    ),
                 ),
-            ),
-        );
+            ))
+            ->build();
 
-        $extension->load(array(array('dbal' => array('connections' => array('default' => array('password' => 'foo'))), 'orm' => $config)), $container);
+        $extension->load(array($config), $container);
 
         $this->assertEquals(AbstractProxyFactory::AUTOGENERATE_EVAL, $container->getParameter('doctrine.orm.auto_generate_proxy_classes'));
     }
 
-    public function testSingleEntityManagerConfiguration()
+    public function testSingleEntityManagerWithDefaultConfiguration()
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
 
-        $extension->load(array(array('dbal' => array('connections' => array('default' => array('password' => 'foo'))), 'orm' => array('default_entity_manager' => 'default', 'entity_managers' => array('default' => array('mappings' => array('YamlBundle' => array())))))), $container);
+        $configurationArray = BundleConfigurationBuilder::createBuilderWithBaseValues()->build();
+
+        $extension->load(array($configurationArray), $container);
         $this->compileContainer($container);
 
         $definition = $container->getDefinition('doctrine.orm.default_entity_manager');
@@ -328,12 +388,87 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         ));
     }
 
+    public function testSingleEntityManagerWithDefaultSecondLevelCacheConfiguration()
+    {
+        if (version_compare(Version::VERSION, "2.5.0-DEV") < 0) {
+            $this->markTestSkipped(sprintf('Second Level cache not supported by this version of the ORM : %s', Version::VERSION));
+        }
+        $container = $this->getContainer();
+        $extension = new DoctrineExtension();
+
+        $configurationArray = BundleConfigurationBuilder::createBuilderWithBaseValues()
+            ->addBaseSecondLevelCache()
+            ->build();
+
+        $extension->load(array($configurationArray), $container);
+        $this->compileContainer($container);
+
+        $definition = $container->getDefinition('doctrine.orm.default_entity_manager');
+        $this->assertEquals('%doctrine.orm.entity_manager.class%', $definition->getClass());
+        if (method_exists($definition, 'getFactory')) {
+            $this->assertEquals(array('%doctrine.orm.entity_manager.class%', 'create'), $definition->getFactory());
+        } else {
+            $this->assertEquals('%doctrine.orm.entity_manager.class%', $definition->getFactoryClass());
+            $this->assertEquals('create', $definition->getFactoryMethod());
+        }
+
+        $this->assertDICConstructorArguments($definition, array(
+            new Reference('doctrine.dbal.default_connection'), new Reference('doctrine.orm.default_configuration'),
+        ));
+
+        $slcDefinition = $container->getDefinition('doctrine.orm.default_second_level_cache.default_cache_factory');
+        $this->assertEquals('%doctrine.orm.second_level_cache.default_cache_factory.class%', $slcDefinition->getClass());
+    }
+
+    public function testSingleEntityManagerWithCustomSecondLevelCacheConfiguration()
+    {
+        if (version_compare(Version::VERSION, "2.5.0-DEV") < 0) {
+            $this->markTestSkipped(sprintf('Second Level cache not supported by this version of the ORM : %s', Version::VERSION));
+        }
+        $container = $this->getContainer();
+        $extension = new DoctrineExtension();
+
+        $configurationArray = BundleConfigurationBuilder::createBuilderWithBaseValues()
+            ->addSecondLevelCache([
+                'region_cache_driver' => [
+                    'type' => 'memcache'],
+                'regions' => [
+                    'hour_region' => [
+                        'lifetime' => 3600
+                    ]
+                ],
+                'factory' => 'YamlBundle\Cache\MyCacheFactory',
+            ])
+            ->build();
+
+        $extension->load(array($configurationArray), $container);
+        $this->compileContainer($container);
+
+        $definition = $container->getDefinition('doctrine.orm.default_entity_manager');
+        $this->assertEquals('%doctrine.orm.entity_manager.class%', $definition->getClass());
+        if (method_exists($definition, 'getFactory')) {
+            $this->assertEquals(array('%doctrine.orm.entity_manager.class%', 'create'), $definition->getFactory());
+        } else {
+            $this->assertEquals('%doctrine.orm.entity_manager.class%', $definition->getFactoryClass());
+            $this->assertEquals('create', $definition->getFactoryMethod());
+        }
+
+        $this->assertDICConstructorArguments($definition, array(
+            new Reference('doctrine.dbal.default_connection'), new Reference('doctrine.orm.default_configuration'),
+        ));
+
+        $slcDefinition = $container->getDefinition('doctrine.orm.default_second_level_cache.default_cache_factory');
+        $this->assertEquals('YamlBundle\Cache\MyCacheFactory', $slcDefinition->getClass());
+    }
+
     public function testBundleEntityAliases()
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
 
-        $config = $this->getConnectionConfig();
+        $config = BundleConfigurationBuilder::createBuilder()
+             ->addBaseConnection()
+             ->build();
         $config['orm'] = array('default_entity_manager' => 'default', 'entity_managers' => array('default' => array('mappings' => array('YamlBundle' => array()))));
         $extension->load(array($config), $container);
 
@@ -348,7 +483,9 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
 
-        $config = $this->getConnectionConfig();
+        $config = BundleConfigurationBuilder::createBuilder()
+             ->addBaseConnection()
+             ->build();
         $config['orm'] = array('default_entity_manager' => 'default', 'entity_managers' => array('default' => array('mappings' => array('YamlBundle' => array('alias' => 'yml')))));
         $extension->load(array($config), $container);
 
@@ -363,8 +500,10 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container = $this->getContainer('YamlBundle');
         $extension = new DoctrineExtension();
 
-        $config = $this->getConnectionConfig();
-        $config['orm'] = array('default_entity_manager' => 'default', 'entity_managers' => array('default' => array('mappings' => array('YamlBundle' => array()))));
+        $config = BundleConfigurationBuilder::createBuilder()
+            ->addBaseConnection()
+            ->addBaseEntityManager()
+            ->build();
         $extension->load(array($config), $container);
 
         $definition = $container->getDefinition('doctrine.orm.default_metadata_driver');
@@ -379,8 +518,19 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container = $this->getContainer('XmlBundle');
         $extension = new DoctrineExtension();
 
-        $config = $this->getConnectionConfig();
-        $config['orm'] = array('default_entity_manager' => 'default', 'entity_managers' => array('default' => array('mappings' => array('XmlBundle' => array()))));
+        $config = BundleConfigurationBuilder::createBuilder()
+            ->addBaseConnection()
+            ->addEntityManager(array(
+                'default_entity_manager' => 'default',
+                'entity_managers' => array(
+                    'default' => array(
+                        'mappings' => array(
+                            'XmlBundle' => array()
+                        )
+                    )
+                )
+            ))
+            ->build();
         $extension->load(array($config), $container);
 
         $definition = $container->getDefinition('doctrine.orm.default_metadata_driver');
@@ -395,8 +545,19 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container = $this->getContainer('AnnotationsBundle');
         $extension = new DoctrineExtension();
 
-        $config = $this->getConnectionConfig();
-        $config['orm'] = array('default_entity_manager' => 'default', 'entity_managers' => array('default' => array('mappings' => array('AnnotationsBundle' => array()))));
+        $config = BundleConfigurationBuilder::createBuilder()
+            ->addBaseConnection()
+            ->addEntityManager(array(
+                'default_entity_manager' => 'default',
+                'entity_managers' => array(
+                    'default' => array(
+                        'mappings' => array(
+                            'AnnotationsBundle' => array()
+                        )
+                    )
+                )
+            ))
+            ->build();
         $extension->load(array($config), $container);
 
         $definition = $container->getDefinition('doctrine.orm.default_metadata_driver');
@@ -411,22 +572,34 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container = $this->getContainer(array('XmlBundle', 'AnnotationsBundle'));
         $extension = new DoctrineExtension();
 
-        $config1 = $this->getConnectionConfig();
-        $config1['orm'] = array(
-            'auto_generate_proxy_classes' => true,
-            'default_entity_manager' => 'default',
-            'entity_managers' => array(
-                'default' => array('mappings' => array('AnnotationsBundle' => array())),
-            ),
-        );
-        $config2 = $this->getConnectionConfig();
-        $config2['orm'] = array(
-            'auto_generate_proxy_classes' => false,
-            'default_entity_manager' => 'default',
-            'entity_managers' => array(
-                'default' => array('mappings' => array('XmlBundle' => array())),
-            ),
-        );
+        $config1 = BundleConfigurationBuilder::createBuilder()
+            ->addBaseConnection()
+            ->addEntityManager(array(
+                'auto_generate_proxy_classes' => true,
+                'default_entity_manager' => 'default',
+                'entity_managers' => array(
+                    'default' => array(
+                        'mappings' => array(
+                            'AnnotationsBundle' => array()
+                        )
+                    ),
+                ),
+            ))
+            ->build();
+        $config2 = BundleConfigurationBuilder::createBuilder()
+            ->addBaseConnection()
+            ->addEntityManager(array(
+                'auto_generate_proxy_classes' => false,
+                'default_entity_manager' => 'default',
+                'entity_managers' => array(
+                    'default' => array(
+                        'mappings' => array(
+                            'XmlBundle' => array()
+                        )
+                    ),
+                ),
+            ))
+            ->build();
         $extension->load(array($config1, $config2), $container);
 
         $definition = $container->getDefinition('doctrine.orm.default_metadata_driver');
@@ -456,8 +629,19 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container = $this->getContainer('AnnotationsBundle', 'Vendor');
         $extension = new DoctrineExtension();
 
-        $config = $this->getConnectionConfig();
-        $config['orm'] = array('default_entity_manager' => 'default', 'entity_managers' => array('default' => array('mappings' => array('AnnotationsBundle' => array()))));
+        $config = BundleConfigurationBuilder::createBuilder()
+            ->addBaseConnection()
+            ->addEntityManager(array(
+                'default_entity_manager' => 'default',
+                'entity_managers' => array(
+                    'default' => array(
+                        'mappings' => array(
+                            'AnnotationsBundle' => array()
+                        )
+                    )
+                )
+            ))
+            ->build();
         $extension->load(array($config), $container);
 
         $calls = $container->getDefinition('doctrine.orm.default_metadata_driver')->getMethodCalls();
@@ -470,18 +654,20 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
 
-        $config = $this->getConnectionConfig();
-        $config['orm'] = array(
-            'metadata_cache_driver' => array(
-                'cache_provider' => 'metadata_cache',
-            ),
-            'query_cache_driver' => array(
-                'cache_provider' => 'query_cache',
-            ),
-            'result_cache_driver' => array(
-                'cache_provider' => 'result_cache',
-            ),
-        );
+        $config = BundleConfigurationBuilder::createBuilder()
+             ->addBaseConnection()
+             ->addEntityManager(array(
+                 'metadata_cache_driver' => array(
+                     'cache_provider' => 'metadata_cache',
+                 ),
+                 'query_cache_driver' => array(
+                     'cache_provider' => 'query_cache',
+                 ),
+                 'result_cache_driver' => array(
+                     'cache_provider' => 'result_cache',
+                 ),
+             ))
+            ->build();
 
         $extension->load(array($config), $container);
 
@@ -498,6 +684,30 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals('doctrine_cache.providers.result_cache', (string) $alias);
     }
 
+    public function testShardManager()
+    {
+        $container = $this->getContainer();
+        $extension = new DoctrineExtension();
+
+        $config = BundleConfigurationBuilder::createBuilder()
+             ->addConnection(array(
+                 'connections' => array(
+                     'foo' => array(
+                         'shards' => array(
+                             'test' => array('id' => 1)
+                         ),
+                     ),
+                     'bar' => array(),
+                 ),
+             ))
+            ->build();
+
+        $extension->load(array($config), $container);
+
+        $this->assertTrue($container->hasDefinition('doctrine.dbal.foo_shard_manager'));
+        $this->assertFalse($container->hasDefinition('doctrine.dbal.bar_shard_manager'));
+    }
+
     private function getContainer($bundles = 'YamlBundle', $vendor = null)
     {
         $bundles = (array) $bundles;
@@ -570,8 +780,6 @@ class DoctrineExtensionTest extends \PHPUnit_Framework_TestCase
         $container->compile();
     }
 
-    private function getConnectionConfig()
-    {
-        return array('dbal' => array('connections' => array('default' => array('password' => 'foo'))));
-    }
 }
+
+class TestWrapperClass extends Connection {}
\ No newline at end of file
